#Screen a dataset for georeferencing errors, incomplete information and 
#redundant occurrence records

if (!require(dplyr)) install.packages("dplyr", dependencies = T)
if (!require(sf)) install.packages("sf", dependencies = T)
if (!require(tmap)) install.packages("tmap", dependencies = T)



#Import dataset

  dataset <- read.csv("./freshw_macroinv_cu.csv")

# 1. Check for georeferencing errors

  # Map of Cuba

  mapCuba <- st_read("https://geodata.ucdavis.edu/gadm/gadm4.1/gpkg/gadm41_CUB.gpkg")

  # Transform records to sf object
  
  pts <- st_as_sf(dataset, coords = c("decimalLongitude","decimalLatitude"))%>%
  st_set_crs(4326)

  # Points located in the ocean
  
  outlandPts <-  pts %>%
    mutate(inland = st_intersects(pts, mapCuba, sparse = F)[,1]) %>%
    filter(inland == F) %>%
    select(scientificName, locality, municipality, stateProvince, inland)
  
  if (nrow(outlandPts) != 0) {
    print("Records located in the ocean")
    st_drop_geometry(outlandPts) %>%
      tibble() %>%
      print()
    
    tm_shape(mapCuba) +
      tm_polygons() +
      tm_shape(outlandPts) +
      tm_symbols(size = 1, 
                 col = "inland") +
      tm_layout(main.title = "Records located in the ocean",
                legend.show = F)
    
    
  } else {
    print("No georeferencing errors detected")
  }

  
# 2. Check for incomplete information

  # Occurrence records with missing taxonomic information
  
  missingTaxName <- dataset %>%
    filter(is.na(scientificName) | 
             is.na(genus) |
             is.na(family) |
             is.na(order) |
             is.na(class) |
             is.na(phylum))
  
  if (nrow(missingTaxName) != 0) {
    print("Records with missing taxonomic information")
    tibble(missingTaxName) %>%
      select(scientificName, genus, family, order, class, phylum, locality, municipality, stateProvince)
    
  } else {
    print("No missing taxonomic information detected")
  }
 
  
  # Occurrence records with missing date information
  
  missingDate <- dataset %>%
    filter(is.na(year))
  
  if (nrow(missingDate) != 0) {
    print("Records with missing year")
    tibble(missingTaxName) %>%
      select(scientificName, locality, municipality, stateProvince, day, month, year)
    
  } else {
    print("No record with missing year detected")
  }
  
  # Occurrence records with missing geographic information
  
  missingGeo <- dataset %>%
    filter(is.na(decimalLongitude) | 
             is.na(decimalLatitude) |
             is.na(locality) |
             is.na(municipality) | 
             is.na(stateProvince) |
             is.na(coordinateUncertaintyInMeters)
             )
  
  
  if (nrow(missingGeo) != 0) {
    print("Records with missing year")
    tibble(missingTaxName) %>%
      select(locality, decimalLongitude, decimalLatitude, municipality, 
             stateProvince, coordinateUncertaintyInMeters)
    
  } else {
    print("No record with missing geographic information detected")
  }
  

# 3. Check for redundant occurrence records and delete repetitions
  
  # Keep only distinct records  
  noRepeat <- dataset %>%
  distinct(scientificName, decimalLatitude, decimalLongitude,
           day, month, year, .keep_all = T)
  
 nCorr <- nrow(dataset) - nrow(noRepeat)

 print(paste0(nCorr, " redundant records were detected"))


